/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.dhcp.control
 * File:    FileWatcher.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.dhcp.control;

import java.io.File;
import java.util.Observable;

/**
 * Monitors a file for changes in a given intervall 
 * @author Tobias
 *
 */
public class FileWatcher extends Observable implements Runnable {

	private String path;
	private long modified = 0;	
	private int intervall = 1; 
		
	/**
	 * Initialize the File-Watcher
	 * @param path  the file that will be observed
	 */
	FileWatcher(String path){
		this.path = path;
	}
	
	/**
	 * The time the next check is performed
	 * @return time in seconds
	 */
	public int getIntervall() {
		return intervall;
	}
	
	/**
	 * Sets up the intervall
	 * @param the intervall in seconds
	 */	
	public void setIntervall(int sec) {
		intervall = sec;
	}
	
	/**
	 * The thread starts here
	 */
	@Override
	public void run() {
		long tmp = 0;
	while(true){
		try {
			do{			
				tmp = this.getLastModified(path);
				if(modified == 0){ 
					modified = tmp; 
					setChanged();
					notifyObservers();
				}
			//	System.out.print('.');
				Thread.sleep(intervall*1000);
			}while(modified == tmp);						
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		setChanged();
		notifyObservers();
		modified = tmp;
		
	 }
	}
	
	/**
	 * Delivers the time when the file is last modified
	 * 
	 * @param path path to file
	 * @return timestamp the file was last modified
	 */
	private long getLastModified(String path){
		File f = new File(path);
		long last = f.lastModified();
		f = null;
		return last;
	}

}
