/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.dhcp.control
 * File:    MapControl.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.dhcp.control;

import java.rmi.RemoteException;
import java.util.Observable;
import java.util.Observer;

import com.martiansoftware.jsap.JSAPException;

import de.fhhannover.inform.iron.dhcp.parser.ArgumentParser;
import de.fhhannover.inform.iron.dhcp.parser.LeaseManager;
import de.fhhannover.inform.iron.dhcp.translator.LeaseDistributor;

/**
 * Entry-Class. Here we initiate all components and stick them together
 * @author Tobias
 *
 */
public class MapControl implements Observer {
	
	private FileWatcher watcher;
	private LeaseDistributor distributor;
	
	
	/**
	 * The default-constructur registers this object to the filewatcher and sets its intervall
	 */
	MapControl(String[] args){
		try {
			ArgumentParser.start(args);
		} catch (JSAPException e1) {			
			e1.printStackTrace();
		}
		PropertiesReader.loadProperties("mapdhcp.conf");
		watcher =  new FileWatcher(Config.DHCPD_LEASES_PATH);		
		watcher.setIntervall(Config.MAPDHCP_PARSER_INTERVAL);
		watcher.addObserver(this);
		distributor = new LeaseDistributor();
		// set the shutdown-hook for send endSession()
		Runtime.getRuntime().addShutdownHook(new Thread() {
		    public void run() { try {		    	
				distributor.sendEndSession();
			} catch (RemoteException e) {
				e.printStackTrace();
			} }
		});
				
	}
	
	/**
	 * Starts the program
	 */
	public void start(){		
		// we send a purgePublish()
		if(!distributor.sendPurgePublish())
			System.err.println("Theres something wrong with purgePublish()");
		// we start the filewatcher-thread
		new Thread(watcher).start();
	}
	
	/**
	 * When a change is detected, we call the Lease-Distributor with the new list
	 */
	@Override
	public void update(Observable o, Object arg) {
		distributor.sendLeases(LeaseManager.getLeases());
	}

	/**
	 * @param args no args yet
	 */
	public static void main(String[] args) {
		MapControl mapdhcp = new MapControl(args);	
		mapdhcp.start();
	}

}
