/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.dhcp.translator
 * File:    LeaseDistributor.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.dhcp.translator;

import java.rmi.RemoteException;
import java.util.ArrayList;
import de.fhhannover.inform.iron.dhcp.parser.Lease;
import de.fhhannover.inform.iron.dhcp.soap.SOAPControl;
/**
 * This class adjusts the new leases with the old ones and outreaches them to the SOAP-component
 * @author Tobias
 *
 */
public class LeaseDistributor {
	
	private ArrayList<Lease> old_leases = new ArrayList<Lease>();
	private ArrayList<Lease> add_leases;
	private ArrayList<Lease> del_leases;
	
	/**
	 * Compares two arrays with leases
	 * @param l1 first array
	 * @param l2 second array
	 * @return an adjusted list of leases
	 * 
	 * @todo
	 */
	private void compareAndFill(ArrayList<Lease> _new, ArrayList<Lease> _old){
		add_leases = new ArrayList<Lease>();
		del_leases = new ArrayList<Lease>();
		//we try to find new leases		
		Lease tmp = null;
		// FIXME
		if( _new==null || _old==null) {
			System.err.println("compareAndFill parameter is null.");
			return;
		}
		for (Lease l1 : _new) {	
			if(l1.getBindingState().equals("active")){
				boolean isOld = false;					
				for(int i = 0; i<_old.size();i++){
					if(l1.getIp().equals(_old.get(i).getIp())){
						isOld = true;
						tmp = _old.get(i);
					}
				}
				if(!isOld){
					add_leases.add(l1);					
				}
				else {					
					if(l1.getEnd().compareTo(tmp.getEnd()) > 0){								
							add_leases.add(l1);	
							old_leases.remove(tmp);
					}
				}
			}
			else if(l1.getBindingState().equals("free")){
				boolean isOld = false;			
				for(int i = 0; i<_old.size();i++){
					if(l1.getIp().equals(_old.get(i).getIp())){
						isOld = true;
						tmp = _old.get(i);
					}
				}
				
				if(isOld){
					del_leases.add(l1);	
					old_leases.remove(tmp);
				}				
			}
		}
		
		// we also delete the outstanding leases on the MAPS
		//for(int i = 0; i<old_leases.size(); i++){
			//del_leases.add(old_leases.get(i));
		//}
		//old_leases.clear();
		
		// we copy the actual leases
	for(int i = 0; i<add_leases.size(); i++){
		old_leases.add(add_leases.get(i));
	} 			
	}
	
	private ArrayList<Lease> filterDuplicates(ArrayList<Lease> list){
		if(list == null || list.size() == 0){
			return list;
		}
		ArrayList<Lease> tmp = new ArrayList<Lease>();
		for(Lease x : list){
			for(int i=0;  i<tmp.size(); i++){
				if(x.getIp().equals(tmp.get(i).getIp())){
					tmp.remove(i);
				}
			}
			tmp.add(x);
		}
		return tmp;
	}
	/**
	 * Passes a lease array to the SOAP-component.
	 * @param new_leases
	 */
	public void sendLeases(ArrayList<Lease> new_leases){
		ArrayList<Lease> no_dups = filterDuplicates(new_leases);
		compareAndFill(no_dups, old_leases);
		System.out.println("NewLeases:");
		print_r(add_leases);
		System.out.println("DelLeases:");
		print_r(del_leases);	
		SOAPControl.sendSOAPMessages(add_leases);
		SOAPControl.deleteSOAPMessages(del_leases);		
	}
	
	public void removeLeases(ArrayList<Lease> new_leases){
		
	}
	
	private void print_r(ArrayList<Lease> list){
		if(list.size() == 0){System.out.println('-');}
		else{
			for(int i=0; i<list.size(); i++){
				System.out.println("IP:"+ list.get(i).getIp()+" MAC:"+list.get(i).getMac());
			}
		}
	}
	
	public boolean sendPurgePublish(){
		return SOAPControl.sendPurgePublisher();
	}
	
	public void sendEndSession() throws RemoteException{
		SOAPControl.endSession();
	}
}
